
//  playcd.c
//
//  CD File Player Using Borland C++ 3.1 MCI Functions
//
//  by Chris Peterson (Compuserve ID 76550,3670)
//
//  Written on May 9, 1993
//
//  Note:  Set Tab Stops to 4


#include <windows.h>
#include <stdio.h>
#include <mmsystem.h>
#include <string.h>

//  Main Menu Defines

#define		ID_RESTART		101		//  Start Playing From Track 1
#define		ID_EXIT			102		//  Exit This Program
#define		ID_ABOUT		103		//  Display About Dialog Box

//  CD Control Defines

#define		ID_PLAY			104		//  Start Playing CD
#define		ID_SKIP			105		//  Skip to Next Track
#define		ID_PREV			106		//  Skip to Previous Track
#define		ID_STOP			107		//  Stop CD Player

void PlayCD(),			//  Start Playing The CD
     RestartCD(),		//  Restart The CD From Track 1
	 SkipTrack(),		//  Skip to The Next Track
	 PrevTrack(),		//  Skip to The Previous Track
	 NumTracks(),		//  Get The Total # of Tracks on The Current CD
	 SetCDParms(),		//  Set CD Parameters
	 StopCD(),			//  Stop Playing The CD
     ShowCdStatus();	//  Display Current CD Status as The Dialog Window Title

int  OpenCD(),			//  Open The CD Device
	 CloseCD();			//  Close The CD Device

int	 IsCdPlaying(),		//  Is The CD Device Currently Playing ?
	 GetCurTrack(),		//  Get The Current Track #
     CheckStatus(int);	//  Check CD Player Status

int  CurTrack  = 1,		//  Current Track #
	 LastTrack = 0,		//  Total # of Tracks on The CD
	 CdIsOpen  = 0;		//  Is The CD Device Currently Open ?


//  MCI Parameter Structures

static MCI_GENERIC_PARMS GenParms;
static MCI_OPEN_PARMS    OpenParms;
static MCI_PLAY_PARMS    PlayParms;
static MCI_SET_PARMS     SetParms;
static MCI_STATUS_PARMS  StatusParms;

WORD wDeviceID;			//  MCI CD Device ID 

HWND hwnd;				//  Main Window Handle


long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);	//  Main Message Processer


int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
	static char szAppName[] = "MCI CD Player";
	MSG         msg;
	WNDCLASS    wndclass;
    DWORD 		WStyle = WS_POPUPWINDOW | WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX;

	if (hPrevInstance == 0)
	{
		wndclass.style         = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc   = WndProc;
		wndclass.cbClsExtra    = 0;
		wndclass.cbWndExtra    = 0;
		wndclass.hInstance     = hInstance;
		wndclass.hIcon         = LoadIcon(hInstance, "CDICON");
		wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
		wndclass.hbrBackground = GetStockObject(WHITE_BRUSH);
		wndclass.lpszMenuName  = "CDMENU";
		wndclass.lpszClassName = szAppName;
		RegisterClass(&wndclass);
	}

	hwnd = CreateWindow(szAppName,          //  window class name
			"MCI CD Player",           		//  window caption
			WStyle,                         //  window style
			150,                            //  initial x position
			100,                            //  initial y position
			305,                            //  initial x size
			120,                            //  initial y size
			NULL,                           //  parent window handle
			NULL,                           //  window menu handle
			hInstance,                      //  program instance handle
			NULL);					        //  creation parameters

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	CreateWindow ("button", "Play", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 20, 25, 50, 35,
				 hwnd, (HMENU)ID_PLAY, hInstance, NULL);
	CreateWindow ("button", "-->>", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 90, 25, 50, 35,
				 hwnd, (HMENU)ID_SKIP, hInstance, NULL);
	CreateWindow ("button", "<<--", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 160, 25, 50, 35,
				 hwnd, (HMENU)ID_PREV, hInstance, NULL);
	CreateWindow ("button", "Stop", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 230, 25, 50, 35,
				 hwnd, (HMENU)ID_STOP, hInstance, NULL);

	SetTimer(hwnd, 1, 3000, NULL);		//  Set Timer Procedure Value (Milliseconds)
	    	
	while (GetMessage(&msg, NULL, 0, 0) != 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return(msg.wParam);
}


long FAR PASCAL _export WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
{
	HDC         hdc;
	PAINTSTRUCT ps;
    int			ct;

	switch (message)
	{
		case WM_CREATE:
			OpenCD();
			if (IsCdPlaying() == 1)
			{
				CurTrack = GetCurTrack();
				ShowCdStatus();
            }
            return(0);
		case WM_PAINT:
			BeginPaint(hwnd, &ps);
			EndPaint(hwnd, &ps);
            ShowCdStatus();
			return(0);

		case WM_COMMAND:
			if (wParam == ID_RESTART)		//  Restart Play From Track 1
			{
				RestartCD();
            }
			if (wParam == ID_EXIT)			//  Exit This Program
			{
				SendMessage(hwnd, WM_DESTROY, NULL, NULL);
            }
			else if (wParam == ID_ABOUT)	//  Display About Box
			{
				MessageBox(hwnd, "by Chris Peterson (CompuServe ID: 76550,3670)", "MCI CD Player", MB_OK);
			}
			else if (wParam == ID_PLAY)
			{
				PlayCD();			//  Start CD Player
			}
			else if (wParam == ID_SKIP)
			{
				SkipTrack();		//  Skip to Next Track
			}
			else if (wParam == ID_PREV)
			{
				PrevTrack();		//  Skip to Previous Track
			}
			else if (wParam == ID_STOP)
			{
				StopCD();			//  Stop CD Player
			}
			return(0);

		case WM_TIMER:
			if (CdIsOpen == 1 && IsCdPlaying() == 1)
            {
				if ((ct = GetCurTrack()) != CurTrack)
				{
					CurTrack = ct;
                    ShowCdStatus();
				}
            }
            return(0);

		case WM_DESTROY:
			KillTimer(hwnd, 1);
            CloseCD();
			PostQuitMessage(0);
			return(0);

	}
	return(DefWindowProc(hwnd, message, wParam, lParam));
}


void ShowCdStatus()		//  Display CD Player Status as Window Title

{   int x;
	static char buf[100];

	if (CdIsOpen == 0)
	{
		SetWindowText(hwnd,"CD Drive is Unavailable");
		return;
	}

	x = CheckStatus(MCI_STATUS_MODE);

	strcpy(buf,"Unknown CD Status");

	switch (x)
	{
		case MCI_MODE_PLAY:
			sprintf(buf, "Playing Track %d of %d", CurTrack, LastTrack);
			break;
		case MCI_MODE_NOT_READY:
			strcpy(buf,"CD Not Ready");
			break;
		case MCI_MODE_PAUSE:
			strcpy(buf,"CD is Paused");
			break;
		case MCI_MODE_STOP:
			strcpy(buf,"CD is Stopped");
			break;
		case MCI_MODE_OPEN:
			strcpy(buf,"CD Door is Open");
			break;
		case MCI_MODE_SEEK:
			strcpy(buf,"CD is Seeking");
			break;
	}

    SetWindowText(hwnd, buf);
}


int IsCdPlaying()		//  See if CD is Currently Playing

{
	if (CheckStatus(MCI_STATUS_MODE) == MCI_MODE_PLAY) return(1);
    return(0);
}


void NumTracks()		//  Get Total # of Tracks on CD

{   int x;

	if ((x = CheckStatus(MCI_STATUS_NUMBER_OF_TRACKS)) > 0) LastTrack = x;
}


int GetCurTrack()		//  Get Current Track #

{   int x;

	if ((x = CheckStatus(MCI_STATUS_CURRENT_TRACK)) > 0) return(x);
    return(0);
}


int CheckStatus(int Value)		//  Check CD Status

{
	static char buf[256];
	int x;

	if (CdIsOpen == 0) return(1);

	StatusParms.dwCallback = 0L;
	StatusParms.dwItem = Value;
	x = mciSendCommand(wDeviceID, MCI_STATUS, MCI_STATUS_ITEM | MCI_WAIT,(DWORD)(LPMCI_STATUS_PARMS)&StatusParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Status Error", MB_OK | MB_ICONEXCLAMATION);
        return(-1);
	}
    return(StatusParms.dwReturn);
}


void RestartCD()		//  Start CD at Track 1
{
	CurTrack = 1;
	PlayCD();
}


void SkipTrack()		//  Skip to Next Track
{
	if (++CurTrack > LastTrack) CurTrack = 1;
    PlayCD();
}

void PrevTrack()		//  Skip to Previous Track
{
	if (--CurTrack == 0) CurTrack = LastTrack;
    PlayCD();
}


int OpenCD()		//  Open The CD Device
{
	static char buf[256];
	int x;

	OpenParms.dwCallback = 0L;
	OpenParms.wDeviceID = 0;
	OpenParms.wReserved0 = 0;
	OpenParms.lpstrDeviceType = (LPSTR)"cdaudio";
	OpenParms.lpstrElementName = NULL;
	OpenParms.lpstrAlias = NULL;
	x = mciSendCommand(0, MCI_OPEN, MCI_WAIT| MCI_OPEN_TYPE,(DWORD)(LPMCI_OPEN_PARMS)&OpenParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Open Error", MB_OK | MB_ICONEXCLAMATION);
        return(1);
	}
	else
    {
		wDeviceID = OpenParms.wDeviceID;
		CdIsOpen = 1;
        SetCDParms();
		NumTracks();
        ShowCdStatus();
		return(0);
    }
}

void SetCDParms()		//  Set CD Time Parameters
{
	static char buf[256];
	int x;

	if (CdIsOpen == 0) return;

	SetParms.dwCallback = 0L;
	SetParms.dwTimeFormat = MCI_FORMAT_TMSF;
	x = mciSendCommand(wDeviceID, MCI_SET, MCI_WAIT| MCI_SET_TIME_FORMAT,(DWORD)(LPMCI_SET_PARMS)&SetParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Set Error", MB_OK | MB_ICONEXCLAMATION);
    }
}


void PlayCD()		//  Start Playing The CD
{
	int x;
	static char buf[256];

	if (CdIsOpen == 0) return;

    StopCD();
	PlayParms.dwCallback = 0L;
	PlayParms.dwFrom = CurTrack;
	x = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY | MCI_FROM ,(DWORD) (LPMCI_PLAY_PARMS)&PlayParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Play Error", MB_OK | MB_ICONEXCLAMATION);
	}
    NumTracks();
    ShowCdStatus();
}

void StopCD()		//  Stop The CD Player
{
	int x;
	static char buf[256];

	if (CdIsOpen == 0) return;

	GenParms.dwCallback = 0L;
	x = mciSendCommand(wDeviceID, MCI_STOP, MCI_WAIT,(DWORD)(LPMCI_GENERIC_PARMS)&GenParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Stop Error", MB_OK | MB_ICONEXCLAMATION);
	}
	ShowCdStatus();
}

int CloseCD()		//  Close The CD Device
{
	int x;
	static char buf[256];

	if (CdIsOpen == 0) return(1);

	x = mciSendCommand(wDeviceID, MCI_CLOSE, MCI_WAIT,(DWORD)(LPMCI_GENERIC_PARMS)&GenParms);
	if (x != 0)
	{
		mciGetErrorString(x, buf, 256);
		MessageBox(hwnd, buf, "MCI Close Error", MB_OK | MB_ICONEXCLAMATION);
	}
    CdIsOpen = 0;
    return(x);
}
